/* Copyright (c) 2009 Flusso B.V.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE. 
*/
USING nl.flusso.stomp.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS nl.flusso.stomp.MessageProducer IMPLEMENTS javax.jms.MessageProducer:
  DEFINE PROTECTED VARIABLE thisSession AS Session NO-UNDO.


  DEFINE PUBLIC PROPERTY destination AS javax.jms.Destination NO-UNDO
    GET.
    PRIVATE SET.

    
  CONSTRUCTOR PROTECTED MessageProducer(jmsSession AS Session, messageDestination AS javax.jms.Destination):
    thisSession = jmsSession.
    destination = messageDestination.
  END CONSTRUCTOR.


  METHOD PUBLIC VOID send(jmsMessage AS javax.jms.Message):
    send(destination, jmsMessage).
  END METHOD.
  

  /* Workaround for OpenEdge limitation: no interface inheritance. */
  METHOD PUBLIC VOID send(jmsMessage AS javax.jms.TextMessage):
    send(CAST(jmsMessage, javax.jms.Message)).
  END METHOD.

  METHOD PUBLIC VOID send(jmsMessage AS javax.jms.BytesMessage):
    send(CAST(jmsMessage, javax.jms.Message)).
  END METHOD.

  METHOD PUBLIC VOID send(messageDestination AS javax.jms.Destination, jmsMessage AS javax.jms.Message):
    DEFINE VARIABLE jmsQueue AS javax.jms.Queue NO-UNDO.
    DEFINE VARIABLE jmsTopic AS javax.jms.Topic NO-UNDO.
    
    jmsQueue = CAST(messageDestination, javax.jms.Queue) NO-ERROR.
    IF NOT ERROR-STATUS:ERROR THEN
      postMessage('queue':U, jmsQueue:queueName, jmsMessage).
    ELSE
    DO:
      jmsTopic = CAST(messageDestination, javax.jms.Topic).
      postMessage('topic':U, jmsTopic:topicName, jmsMessage).
    END.
  END METHOD.


  /* Workaround for OpenEdge limitation: no interface inheritance. */
  METHOD PUBLIC VOID send(messageDestination AS javax.jms.Destination, jmsMessage AS javax.jms.TextMessage).
    send(messageDestination, CAST(jmsMessage, javax.jms.Message)).
  END METHOD.

  /* Workaround for OpenEdge limitation: no interface inheritance. */
  METHOD PUBLIC VOID send(messageDestination AS javax.jms.Destination, jmsMessage AS javax.jms.BytesMessage).
    send(messageDestination, CAST(jmsMessage, javax.jms.Message)).
  END METHOD.

  /* Workaround for OpenEdge limitation: no interface inheritance. */
  METHOD PUBLIC VOID send(messageDestination AS javax.jms.Queue, jmsMessage AS javax.jms.Message):
    postMessage('queue':U, messageDestination:queueName, jmsMessage).
  END METHOD.


  /* Workaround for OpenEdge limitation: no interface inheritance. */
  METHOD PUBLIC VOID send(messageDestination AS javax.jms.Queue, jmsMessage AS javax.jms.TextMessage):
    send(messageDestination, CAST(jmsMessage, javax.jms.Message)).
  END METHOD.

  /* Workaround for OpenEdge limitation: no interface inheritance. */
  METHOD PUBLIC VOID send(messageDestination AS javax.jms.Queue, jmsMessage AS javax.jms.BytesMessage):
    send(messageDestination, CAST(jmsMessage, javax.jms.Message)).
  END METHOD.

  /* Workaround for OpenEdge limitation: no interface inheritance. */
  METHOD PUBLIC VOID send(messageDestination AS javax.jms.Topic, jmsMessage AS javax.jms.Message):
    postMessage('topic':U, messageDestination:topicName, jmsMessage).
  END METHOD.
  
  
  /* Workaround for OpenEdge limitation: no interface inheritance. */
  METHOD PUBLIC VOID send(messageDestination AS javax.jms.Topic, jmsMessage AS javax.jms.TextMessage).
    send(messageDestination, CAST(jmsMessage, javax.jms.Message)).
  END METHOD.
    
  /* Workaround for OpenEdge limitation: no interface inheritance. */
  METHOD PUBLIC VOID send(messageDestination AS javax.jms.Topic, jmsMessage AS javax.jms.BytesMessage).
    send(messageDestination, CAST(jmsMessage, javax.jms.Message)).
  END METHOD.

  METHOD PUBLIC VOID postMessage(destinationType AS CHARACTER, destinationName AS CHARACTER, jmsMessage AS javax.jms.Message):
    DEFINE VARIABLE aMessage    AS Message    NO-UNDO.
    DEFINE VARIABLE frameToSend AS StompFrame NO-UNDO.

    aMessage = CAST(jmsMessage, Message) NO-ERROR.
    IF NOT ERROR-STATUS:ERROR THEN
    DO:
      frameToSend = aMessage:stompFrame.
      frameToSend:frameType = 'SEND':U.
    END.
    ELSE
      frameToSend = NEW StompFrame(jmsMessage).
    frameToSend:assignHeaderValue('destination':U, SUBSTITUTE('/&1/&2':U, destinationType, destinationName)).
    thisSession:sendFrame(frameToSend).
  END METHOD.
END CLASS.
